@@ -55,7 +55,6 @@ bool libvisio::VDXParser::parseMain()
   if (!m_input)
     return false;
 
-  WPXInputStream *tmpInput = 0;
   try
   {
     std::vector<std::map<unsigned, XForm> > groupXFormsSequence;
@@ -80,8 +79,6 @@ bool libvisio::VDXParser::parseMain()
   }
   catch (...)
   {
-    if (tmpInput)
-      delete tmpInput;
     return false;
   }
 }
@@ -215,9 +212,10 @@ void libvisio::VDXParser::processXmlNode(xmlTextReaderPtr reader)
   case XML_SOLUTIONXML:
     if (XML_READER_TYPE_ELEMENT == tokenType)
     {
+      int ret = 0;
       do
       {
-        xmlTextReaderRead(reader);
+        ret = xmlTextReaderRead(reader);
 #if 0
         // SolutionXML inside VDX file can have invalid namespace URIs
         xmlResetLastError();
@@ -225,7 +223,7 @@ void libvisio::VDXParser::processXmlNode(xmlTextReaderPtr reader)
         tokenId = getElementToken(reader);
         tokenType = xmlTextReaderNodeType(reader);
       }
-      while (XML_SOLUTIONXML != tokenId || XML_READER_TYPE_END_ELEMENT != tokenType);
+      while ((XML_SOLUTIONXML != tokenId || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
     }
     break;
   case XML_STYLESHEET:
@@ -253,6 +251,10 @@ void libvisio::VDXParser::processXmlNode(xmlTextReaderPtr reader)
     if (XML_READER_TYPE_ELEMENT == tokenType)
       readXFormData(reader);
     break;
+  case XML_MISC:
+    if (XML_READER_TYPE_ELEMENT == tokenType)
+      readMisc(reader);
+    break;
   case XML_GEOM:
     if (XML_READER_TYPE_ELEMENT == tokenType)
       readGeometry(reader);
@@ -342,7 +344,7 @@ void libvisio::VDXParser::readLine(xmlTextReaderPtr reader)
       break;
     case XML_LINECOLOR:
       if (XML_READER_TYPE_ELEMENT == tokenType)
-        readExtendedColourData(colour, reader);
+        ret = readExtendedColourData(colour, reader);
       break;
     case XML_LINEPATTERN:
       if (XML_READER_TYPE_ELEMENT == tokenType)
@@ -465,6 +467,33 @@ void libvisio::VDXParser::readFillAndShadow(xmlTextReaderPtr reader)
   }
 }
 
+void libvisio::VDXParser::readMisc(xmlTextReaderPtr reader)
+{
+  int ret = 1;
+  int tokenId = XML_TOKEN_INVALID;
+  int tokenType = -1;
+  do
+  {
+    ret = xmlTextReaderRead(reader);
+    tokenId = getElementToken(reader);
+    if (XML_TOKEN_INVALID == tokenId)
+    {
+      VSD_DEBUG_MSG(("VDXParser::readMisc: unknown token %s\n", xmlTextReaderConstName(reader)));
+    }
+    tokenType = xmlTextReaderNodeType(reader);
+    switch (tokenId)
+    {
+    case XML_HIDETEXT:
+      if (XML_READER_TYPE_ELEMENT == tokenType)
+        ret = readBoolData(m_shape.m_misc.m_hideText, reader);
+      break;
+    default:
+      break;
+    }
+  }
+  while ((XML_MISC != tokenId || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
+}
+
 void libvisio::VDXParser::readXFormData(xmlTextReaderPtr reader)
 {
   int ret = 1;
@@ -783,17 +812,6 @@ void libvisio::VDXParser::readTextBlock(xmlTextReaderPtr reader)
   }
   while ((XML_TEXTBLOCK != tokenId || XML_READER_TYPE_END_ELEMENT != tokenType) && 1 == ret);
 
-  if (bgClrId < 0)
-    bgClrId = 0;
-  if (bgClrId)
-  {
-    std::map<unsigned, Colour>::const_iterator iter = m_colours.find(bgClrId-1);
-    if (iter != m_colours.end())
-      bgColour = iter->second;
-    else
-      bgColour = Colour();
-  }
-
   if (m_isInStyles)
     m_collector->collectTextBlockStyle(level, leftMargin, rightMargin, topMargin, bottomMargin,
                                        verticalAlign, !!bgClrId, bgColour, defaultTabStop, textDirection);
@@ -830,8 +848,8 @@ int libvisio::VDXParser::getElementDepth(xmlTextReaderPtr reader)
 
 void libvisio::VDXParser::getBinaryData(xmlTextReaderPtr reader)
 {
-  xmlTextReaderRead(reader);
-  if (XML_READER_TYPE_TEXT == xmlTextReaderNodeType(reader))
+  const int ret = xmlTextReaderRead(reader);
+  if (1 == ret && XML_READER_TYPE_TEXT == xmlTextReaderNodeType(reader))
   {
     const xmlChar *data = xmlTextReaderConstValue(reader);
     if (data)
