@@ -46,13 +46,26 @@ libvisio::VSDParser::VSDParser(WPXInputStream *input, libwpg::WPGPaintInterface
     m_stencils(), m_currentStencil(0), m_shape(), m_isStencilStarted(false), m_isInStyles(false),
     m_currentShapeLevel(0), m_currentShapeID(MINUS_ONE), m_extractStencils(false), m_colours(),
     m_isBackgroundPage(false), m_isShapeStarted(false), m_shadowOffsetX(0.0), m_shadowOffsetY(0.0),
-    m_currentGeometryList(0), m_currentGeomListCount(0), m_fonts()
+    m_currentGeometryList(0), m_currentGeomListCount(0), m_fonts(), m_names(), m_namesMapMap(),
+    m_currentPageName()
 {}
 
 libvisio::VSDParser::~VSDParser()
 {
 }
 
+void libvisio::VSDParser::_nameFromId(VSDName &name, unsigned id, unsigned level)
+{
+  name = VSDName();
+  std::map<unsigned, std::map<unsigned, VSDName> >::const_iterator iter1 = m_namesMapMap.find(level);
+  if (iter1 != m_namesMapMap.end())
+  {
+    std::map<unsigned, VSDName>::const_iterator iter = iter1->second.find(id);
+    if (iter != iter1->second.end())
+      name = iter->second;
+  }
+}
+
 bool libvisio::VSDParser::getChunkHeader(WPXInputStream *input)
 {
   unsigned char tmpChar = 0;
@@ -196,6 +209,8 @@ void libvisio::VSDParser::handleStreams(WPXInputStream *input, unsigned ptrType,
   std::vector<unsigned> pointerOrder;
   std::map<unsigned, libvisio::Pointer> PtrList;
   std::map<unsigned, libvisio::Pointer> FontFaces;
+  std::map<unsigned, libvisio::Pointer> NameList;
+  std::map<unsigned, libvisio::Pointer> NameIDX;
 
   try
   {
@@ -209,6 +224,10 @@ void libvisio::VSDParser::handleStreams(WPXInputStream *input, unsigned ptrType,
       readPointer(input, ptr);
       if (ptr.Type == VSD_FONTFACES)
         FontFaces[i] = ptr;
+      else if (ptr.Type == VSD_NAME_LIST2)
+        NameList[i] = ptr;
+      else if (ptr.Type == VSD_NAMEIDX || ptr.Type == VSD_NAMEIDX123)
+        NameIDX[i] = ptr;
       else if (ptr.Type != 0)
         PtrList[i] = ptr;
     }
@@ -222,9 +241,16 @@ void libvisio::VSDParser::handleStreams(WPXInputStream *input, unsigned ptrType,
     pointerOrder.clear();
     PtrList.clear();
     FontFaces.clear();
+    NameList.clear();
   }
 
   std::map<unsigned, libvisio::Pointer>::iterator iter;
+  for (iter = NameList.begin(); iter != NameList.end(); ++iter)
+    handleStream(iter->second, iter->first, level+1);
+
+  for (iter = NameIDX.begin(); iter != NameIDX.end(); ++iter)
+    handleStream(iter->second, iter->first, level+1);
+
   for (iter = FontFaces.begin(); iter != FontFaces.end(); ++iter)
     handleStream(iter->second, iter->first, level+1);
 
@@ -274,6 +300,7 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
       m_isBackgroundPage = true;
     else
       m_isBackgroundPage = false;
+    _nameFromId(m_currentPageName, idx, level+1);
     m_collector->startPage(idx);
     break;
   case VSD_STENCILS:
@@ -287,6 +314,7 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
     if (m_extractStencils)
     {
       m_isBackgroundPage = false;
+      _nameFromId(m_currentPageName, idx, level+1);
       m_collector->startPage(idx);
     }
     else
@@ -309,8 +337,7 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
 
   if ((ptr.Format >> 4) == 0x4 || (ptr.Format >> 4) == 0x5 || (ptr.Format >> 4) == 0x0)
   {
-    if (ptr.Length > 4)
-      handleBlob(&tmpInput, shift, level+1);
+    handleBlob(&tmpInput, shift, level+1);
     if ((ptr.Format >> 4) == 0x5 && ptr.Type != VSD_COLORS)
       handleStreams(&tmpInput, ptr.Type, shift, level+1);
   }
@@ -342,7 +369,7 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
     _handleLevelChange(0);
     if (m_extractStencils)
       m_collector->endPage();
-    else
+    else if (m_currentStencil)
     {
       m_stencils.addStencil(idx, *m_currentStencil);
       m_currentStencil = 0;
@@ -355,7 +382,8 @@ void libvisio::VSDParser::handleStream(const Pointer &ptr, unsigned idx, unsigne
     if (m_isStencilStarted)
     {
       _handleLevelChange(0);
-      m_currentStencil->addStencilShape(m_shape.m_shapeId, m_shape);
+      if (m_currentStencil)
+        m_currentStencil->addStencilShape(m_shape.m_shapeId, m_shape);
     }
     break;
   default:
@@ -473,6 +501,12 @@ void libvisio::VSDParser::handleChunk(WPXInputStream *input)
   case VSD_OLE_DATA:
     readOLEData(input);
     break;
+  case VSD_NAMEIDX:
+    readNameIDX(input);
+    break;
+  case VSD_NAMEIDX123:
+    readNameIDX123(input);
+    break;
   case VSD_PAGE_PROPS:
     readPageProps(input);
     break;
@@ -513,9 +547,15 @@ void libvisio::VSDParser::handleChunk(WPXInputStream *input)
   case VSD_NAME_LIST:
     readNameList(input);
     break;
+  case VSD_NAME_LIST2:
+    readNameList2(input);
+    break;
   case VSD_NAME:
     readName(input);
     break;
+  case VSD_NAME2:
+    readName2(input);
+    break;
   case VSD_FIELD_LIST:
     readFieldList(input);
     break;
@@ -534,6 +574,9 @@ void libvisio::VSDParser::handleChunk(WPXInputStream *input)
   case VSD_FONTFACE: // substreams of FONTAFACES stream, ver 11 only
     readFont(input);
     break;
+  case VSD_MISC:
+    readMisc(input);
+    break;
   default:
     m_collector->collectUnhandledChunk(m_header.id, m_header.level);
   }
@@ -550,6 +593,8 @@ void libvisio::VSDParser::_flushShape()
 
   m_collector->collectXFormData(m_currentShapeLevel+2, m_shape.m_xform);
 
+  m_collector->collectMisc(m_currentShapeLevel+2, m_shape.m_misc);
+
   if (m_shape.m_txtxform)
     m_collector->collectTxtXForm(m_currentShapeLevel+2, *(m_shape.m_txtxform));
 
@@ -580,7 +625,7 @@ void libvisio::VSDParser::_flushShape()
   for (std::map<unsigned, VSDName>::const_iterator iterName = m_shape.m_names.begin(); iterName != m_shape.m_names.end(); ++iterName)
     m_collector->collectName(iterName->first, m_currentShapeLevel+2, iterName->second.m_data, iterName->second.m_format);
 
-  if (m_shape.m_foreign)
+  if (m_shape.m_foreign && m_shape.m_foreign->data.size())
     m_collector->collectForeignData(m_currentShapeLevel+1, m_shape.m_foreign->data);
 
   if (!m_shape.m_fields.empty())
@@ -626,6 +671,7 @@ void libvisio::VSDParser::_handleLevelChange(unsigned level)
     {
       _flushShape();
       m_shape.clear();
+      m_currentGeometryList = 0;
     }
     m_isShapeStarted = false;
     m_currentShapeLevel = 0;
@@ -650,7 +696,8 @@ void libvisio::VSDParser::readEllipticalArcTo(WPXInputStream *input)
   input->seek(1, WPX_SEEK_CUR);
   double ecc = readDouble(input); // Eccentricity
 
-  m_currentGeometryList->addEllipticalArcTo(m_header.id, m_header.level, x3, y3, x2, y2, angle, ecc);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addEllipticalArcTo(m_header.id, m_header.level, x3, y3, x2, y2, angle, ecc);
 }
 
 
@@ -687,6 +734,43 @@ void libvisio::VSDParser::readOLEData(WPXInputStream *input)
 
 }
 
+void libvisio::VSDParser::readNameIDX(WPXInputStream *input)
+{
+  std::map<unsigned, VSDName> names;
+  unsigned recordCount = readU32(input);
+  for (unsigned i = 0; i < recordCount; ++i)
+  {
+    unsigned nameId = readU32(input);
+    if (nameId != readU32(input))
+    {
+      VSD_DEBUG_MSG(("VSDParser::readNameIDX --> mismatch of first two dwords\n"));
+      return;
+    }
+    unsigned elementId = readU32(input);
+    input->seek(1, WPX_SEEK_CUR);
+    std::map<unsigned, VSDName>::const_iterator iter = m_names.find(nameId);
+    if (iter != m_names.end())
+      names[elementId] = iter->second;
+  }
+  m_namesMapMap[m_header.level] = names;
+}
+
+void libvisio::VSDParser::readNameIDX123(WPXInputStream *input)
+{
+  std::map<unsigned, VSDName> names;
+  long endPosition = input->tell() + m_header.dataLength;
+  while (!input->atEOS() && input->tell() < endPosition)
+  {
+    unsigned nameId = getUInt(input);
+    unsigned elementId = getUInt(input);
+    std::map<unsigned, VSDName>::const_iterator iter = m_names.find(nameId);
+    if (iter != m_names.end())
+      names[elementId] = iter->second;
+  }
+  m_namesMapMap[m_header.level] = names;
+
+}
+
 void libvisio::VSDParser::readEllipse(WPXInputStream *input)
 {
   input->seek(1, WPX_SEEK_CUR);
@@ -702,7 +786,8 @@ void libvisio::VSDParser::readEllipse(WPXInputStream *input)
   input->seek(1, WPX_SEEK_CUR);
   double ytop = readDouble(input);
 
-  m_currentGeometryList->addEllipse(m_header.id, m_header.level, cx, cy, xleft, yleft, xtop, ytop);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addEllipse(m_header.id, m_header.level, cx, cy, xleft, yleft, xtop, ytop);
 }
 
 void libvisio::VSDParser::readLine(WPXInputStream *input)
@@ -761,6 +846,9 @@ void libvisio::VSDParser::readGeomList(WPXInputStream *input)
 {
   if (!m_shape.m_geometries.empty() && m_currentGeometryList->empty())
     m_shape.m_geometries.erase(--m_currentGeomListCount);
+  // Since this is a map, this will default construct an element and then
+  // the m_currentGeometryList pointer takes its address and we will work
+  // on it over that pointer.
   m_currentGeometryList = &m_shape.m_geometries[m_currentGeomListCount++];
 
   if (m_header.trailer)
@@ -773,7 +861,8 @@ void libvisio::VSDParser::readGeomList(WPXInputStream *input)
     for (unsigned i = 0; i < (childrenListLength / sizeof(uint32_t)); i++)
       geometryOrder.push_back(readU32(input));
 
-    m_currentGeometryList->setElementsOrder(geometryOrder);
+    if (m_currentGeometryList)
+      m_currentGeometryList->setElementsOrder(geometryOrder);
   }
 
   // We want the collectors to still get the level information
@@ -820,7 +909,6 @@ void libvisio::VSDParser::readParaList(WPXInputStream *input)
 
     m_shape.m_paraList.setElementsOrder(paragraphOrder);
   }
-
 }
 
 void libvisio::VSDParser::readPropList(WPXInputStream * /* input */)
@@ -831,7 +919,7 @@ void libvisio::VSDParser::readPage(WPXInputStream *input)
 {
   input->seek(8, WPX_SEEK_CUR); //sub header length and children list length
   unsigned backgroundPageID = readU32(input);
-  m_collector->collectPage(m_header.id, m_header.level, backgroundPageID, m_isBackgroundPage);
+  m_collector->collectPage(m_header.id, m_header.level, backgroundPageID, m_isBackgroundPage, m_currentPageName);
 }
 
 void libvisio::VSDParser::readGeometry(WPXInputStream *input)
@@ -841,7 +929,8 @@ void libvisio::VSDParser::readGeometry(WPXInputStream *input)
   bool noLine = (!!(geomFlags & 2));
   bool noShow = (!!(geomFlags & 4));
 
-  m_currentGeometryList->addGeometry(m_header.id, m_header.level, noFill, noLine, noShow);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addGeometry(m_header.id, m_header.level, noFill, noLine, noShow);
 }
 
 void libvisio::VSDParser::readMoveTo(WPXInputStream *input)
@@ -851,7 +940,8 @@ void libvisio::VSDParser::readMoveTo(WPXInputStream *input)
   input->seek(1, WPX_SEEK_CUR);
   double y = readDouble(input);
 
-  m_currentGeometryList->addMoveTo(m_header.id, m_header.level, x, y);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addMoveTo(m_header.id, m_header.level, x, y);
 }
 
 void libvisio::VSDParser::readLineTo(WPXInputStream *input)
@@ -861,7 +951,8 @@ void libvisio::VSDParser::readLineTo(WPXInputStream *input)
   input->seek(1, WPX_SEEK_CUR);
   double y = readDouble(input);
 
-  m_currentGeometryList->addLineTo(m_header.id, m_header.level, x, y);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addLineTo(m_header.id, m_header.level, x, y);
 }
 
 void libvisio::VSDParser::readArcTo(WPXInputStream *input)
@@ -873,7 +964,8 @@ void libvisio::VSDParser::readArcTo(WPXInputStream *input)
   input->seek(1, WPX_SEEK_CUR);
   double bow = readDouble(input);
 
-  m_currentGeometryList->addArcTo(m_header.id, m_header.level, x2, y2, bow);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addArcTo(m_header.id, m_header.level, x2, y2, bow);
 }
 
 void libvisio::VSDParser::readXFormData(WPXInputStream *input)
@@ -991,7 +1083,7 @@ void libvisio::VSDParser::readPageProps(WPXInputStream *input)
   input->seek(1, WPX_SEEK_CUR);
   scale /= readDouble(input);
 
-  if (m_isStencilStarted)
+  if (m_isStencilStarted && m_currentStencil)
   {
     m_currentStencil->m_shadowOffsetX = m_shadowOffsetX;
     m_currentStencil->m_shadowOffsetY = m_shadowOffsetY;
@@ -1034,6 +1126,7 @@ void libvisio::VSDParser::readShape(WPXInputStream *input)
   }
 
   m_shape.clear();
+  m_currentGeometryList = 0;
   const VSDShape *tmpShape = m_stencils.getStencilShape(masterPage, masterShape);
   if (tmpShape)
   {
@@ -1041,6 +1134,7 @@ void libvisio::VSDParser::readShape(WPXInputStream *input)
       m_shape.m_foreign = new ForeignData(*(tmpShape->m_foreign));
     m_shape.m_text = tmpShape->m_text;
     m_shape.m_textFormat = tmpShape->m_textFormat;
+    m_shape.m_misc = tmpShape->m_misc;
   }
 
   m_shape.m_lineStyleId = lineStyle;
@@ -1072,7 +1166,8 @@ void libvisio::VSDParser::readNURBSTo(WPXInputStream *input)
     input->seek(3, WPX_SEEK_CUR);
     unsigned dataId = readU32(input);
 
-    m_currentGeometryList->addNURBSTo(m_header.id, m_header.level, x, y, knot, knotPrev, weight, weightPrev, dataId);
+    if (m_currentGeometryList)
+      m_currentGeometryList->addNURBSTo(m_header.id, m_header.level, x, y, knot, knotPrev, weight, weightPrev, dataId);
     return;
   }
 
@@ -1147,7 +1242,7 @@ void libvisio::VSDParser::readNURBSTo(WPXInputStream *input)
     unsigned long bytesRead = input->tell() - inputPos;
     unsigned char flag = 0;
     if (paramType != 0x8a) flag = readU8(input);
-    while ((flag != 0x81 || (paramType == 0x8a && repetitions > 0)) && bytesRead < length)
+    while ((paramType == 0x8a ? repetitions > 0 : flag != 0x81) && bytesRead < length)
     {
       inputPos = input->tell();
       double knot_ = 0;
@@ -1200,12 +1295,14 @@ void libvisio::VSDParser::readNURBSTo(WPXInputStream *input)
     knotVector.push_back(lastKnot);
     weights.push_back(weight);
 
-    m_currentGeometryList->addNURBSTo(m_header.id, m_header.level, x, y, xType,
-                                      yType, degree, controlPoints, knotVector, weights);
+    if (m_currentGeometryList)
+      m_currentGeometryList->addNURBSTo(m_header.id, m_header.level, x, y, xType,
+                                        yType, degree, controlPoints, knotVector, weights);
   }
   else // No formula found, use line
   {
-    m_currentGeometryList->addLineTo(m_header.id, m_header.level, x,  y);
+    if (m_currentGeometryList)
+      m_currentGeometryList->addLineTo(m_header.id, m_header.level, x,  y);
   }
 }
 
@@ -1224,7 +1321,8 @@ void libvisio::VSDParser::readPolylineTo(WPXInputStream *input)
     input->seek(3, WPX_SEEK_CUR);
     unsigned dataId = readU32(input);
 
-    m_currentGeometryList->addPolylineTo(m_header.id, m_header.level, x, y, dataId);
+    if (m_currentGeometryList)
+      m_currentGeometryList->addPolylineTo(m_header.id, m_header.level, x, y, dataId);
     return;
   }
 
@@ -1296,12 +1394,14 @@ void libvisio::VSDParser::readPolylineTo(WPXInputStream *input)
       blockBytesRead += input->tell() - inputPos;
     }
 
-    m_currentGeometryList->addPolylineTo(m_header.id, m_header.level, x, y, xType,
-                                         yType, points);
+    if (m_currentGeometryList)
+      m_currentGeometryList->addPolylineTo(m_header.id, m_header.level, x, y, xType,
+                                           yType, points);
   }
   else
   {
-    m_currentGeometryList->addLineTo(m_header.id, m_header.level, x, y);
+    if (m_currentGeometryList)
+      m_currentGeometryList->addLineTo(m_header.id, m_header.level, x, y);
   }
 }
 
@@ -1315,7 +1415,8 @@ void libvisio::VSDParser::readInfiniteLine(WPXInputStream *input)
   double x2 = readDouble(input);
   input->seek(1, WPX_SEEK_CUR);
   double y2 = readDouble(input);
-  m_currentGeometryList->addInfiniteLine(m_header.id, m_header.level, x1, y1, x2, y2);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addInfiniteLine(m_header.id, m_header.level, x1, y1, x2, y2);
 }
 
 void libvisio::VSDParser::readShapeData(WPXInputStream *input)
@@ -1394,7 +1495,8 @@ void libvisio::VSDParser::readSplineStart(WPXInputStream *input)
   double lastKnot = readDouble(input);
   unsigned degree = readU8(input);
 
-  m_currentGeometryList->addSplineStart(m_header.id, m_header.level, x, y, secondKnot, firstKnot, lastKnot, degree);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addSplineStart(m_header.id, m_header.level, x, y, secondKnot, firstKnot, lastKnot, degree);
 }
 
 void libvisio::VSDParser::readSplineKnot(WPXInputStream *input)
@@ -1405,7 +1507,8 @@ void libvisio::VSDParser::readSplineKnot(WPXInputStream *input)
   double y = readDouble(input);
   double knot = readDouble(input);
 
-  m_currentGeometryList->addSplineKnot(m_header.id, m_header.level, x, y, knot);
+  if (m_currentGeometryList)
+    m_currentGeometryList->addSplineKnot(m_header.id, m_header.level, x, y, knot);
 }
 
 void libvisio::VSDParser::readNameList(WPXInputStream * /* input */)
@@ -1413,6 +1516,11 @@ void libvisio::VSDParser::readNameList(WPXInputStream * /* input */)
   m_shape.m_names.clear();
 }
 
+void libvisio::VSDParser::readNameList2(WPXInputStream * /* input */)
+{
+  m_names.clear();
+}
+
 void libvisio::VSDParser::readFieldList(WPXInputStream *input)
 {
   if (m_header.trailer)
@@ -1488,6 +1596,9 @@ void libvisio::VSDParser::readFontIX(WPXInputStream *input)
   case 0: // ANSI
     format = libvisio::VSD_TEXT_ANSI;
     break;
+  case 0x02:
+    format = libvisio::VSD_TEXT_SYMBOL;
+    break;
   case 0xa1: // GREEK
     format = libvisio::VSD_TEXT_GREEK;
     break;
@@ -1515,6 +1626,18 @@ void libvisio::VSDParser::readFontIX(WPXInputStream *input)
   case 0xee: // CENTRAL EUROPE
     format = libvisio::VSD_TEXT_CENTRAL_EUROPE;
     break;
+  case 0x80: // SHIFTJIS
+    format = libvisio::VSD_TEXT_JAPANESE;
+    break;
+  case 0x81: // HANGUL
+    format = libvisio::VSD_TEXT_KOREAN;
+    break;
+  case 0x86: // GB2312
+    format = libvisio::VSD_TEXT_CHINESE_SIMPLIFIED;
+    break;
+  case 0x88: // CHINESEBIG5
+    format = libvisio::VSD_TEXT_CHINESE_TRADITIONAL;
+    break;
   default:
     break;
   }
@@ -1557,9 +1680,10 @@ void libvisio::VSDParser::readText(WPXInputStream *input)
     }
     textStream.append(tmpBuffer, numBytesRead);
     m_shape.m_text = textStream;
-    m_shape.m_textFormat = libvisio::VSD_TEXT_UTF16;
   }
-
+  else
+    m_shape.m_text.clear();
+  m_shape.m_textFormat = libvisio::VSD_TEXT_UTF16;
 }
 
 void libvisio::VSDParser::readCharIX(WPXInputStream *input)
@@ -1668,27 +1792,48 @@ void libvisio::VSDParser::readParaIX(WPXInputStream *input)
 
 void libvisio::VSDParser::readFillAndShadow(WPXInputStream *input)
 {
-  Colour colourFG = _colourFromIndex(readU8(input));
+  unsigned char colourFGIndex = readU8(input);
+  Colour colourFG;
   colourFG.r = readU8(input);
   colourFG.g = readU8(input);
   colourFG.b = readU8(input);
   colourFG.a = readU8(input);
-  double fillFGTransparency = (double)colourFG.a / 255.0;
-  Colour colourBG = _colourFromIndex(readU8(input));
+  unsigned char colourBGIndex = readU8(input);
+  Colour colourBG;
   colourBG.r = readU8(input);
   colourBG.g = readU8(input);
   colourBG.b = readU8(input);
   colourBG.a = readU8(input);
+  if (!colourFG && !colourBG)
+  {
+    colourFG = _colourFromIndex(colourFGIndex);
+    colourBG = _colourFromIndex(colourBGIndex);
+  }
+  double fillFGTransparency = (double)colourFG.a / 255.0;
   double fillBGTransparency = (double)colourBG.a / 255.0;
+
   unsigned char fillPattern = readU8(input);
-  input->seek(1, WPX_SEEK_CUR);
-  Colour shfgc;            // Shadow Foreground Colour
-  shfgc.r = readU8(input);
-  shfgc.g = readU8(input);
-  shfgc.b = readU8(input);
-  shfgc.a = readU8(input);
-  input->seek(5, WPX_SEEK_CUR); // Shadow Background Colour skipped
+
+  unsigned char shadowFGIndex = readU8(input);
+  Colour shadowFG;
+  shadowFG.r = readU8(input);
+  shadowFG.g = readU8(input);
+  shadowFG.b = readU8(input);
+  shadowFG.a = readU8(input);
+  unsigned char shadowBGIndex = readU8(input);
+  Colour shadowBG;
+  shadowBG.r = readU8(input);
+  shadowBG.g = readU8(input);
+  shadowBG.b = readU8(input);
+  shadowBG.a = readU8(input);
+  if (!shadowFG && !shadowBG)
+  {
+    shadowFG = _colourFromIndex(shadowFGIndex);
+    shadowBG = _colourFromIndex(shadowBGIndex);
+  }
+
   unsigned char shadowPattern = readU8(input);
+
 // only version 11 after that point
   input->seek(2, WPX_SEEK_CUR); // Shadow Type and Value format byte
   double shadowOffsetX = readDouble(input);
@@ -1699,7 +1844,7 @@ void libvisio::VSDParser::readFillAndShadow(WPXInputStream *input)
 
   if (m_isInStyles)
     m_collector->collectFillStyle(m_header.level, colourFG, colourBG, fillPattern,
-                                  fillFGTransparency, fillBGTransparency, shadowPattern, shfgc,
+                                  fillFGTransparency, fillBGTransparency, shadowPattern, shadowFG,
                                   shadowOffsetX, shadowOffsetY);
   else
   {
@@ -1708,7 +1853,7 @@ void libvisio::VSDParser::readFillAndShadow(WPXInputStream *input)
       VSD_DEBUG_MSG(("Found stencil fill\n"));
     }
     m_shape.m_fillStyle.override(VSDOptionalFillStyle(colourFG, colourBG, fillPattern, fillFGTransparency,
-                                 fillBGTransparency, shfgc, shadowPattern, shadowOffsetX, shadowOffsetY));
+                                 fillBGTransparency, shadowFG, shadowPattern, shadowOffsetX, shadowOffsetY));
   }
 }
 
@@ -1723,6 +1868,21 @@ void libvisio::VSDParser::readName(WPXInputStream *input)
   }
 }
 
+void libvisio::VSDParser::readName2(WPXInputStream *input)
+{
+  unsigned short unicharacter = 0;
+  ::WPXBinaryData name;
+  input->seek(4, WPX_SEEK_CUR); // skip a dword that seems to be always 1
+  while ((unicharacter = readU16(input)))
+  {
+    name.append(unicharacter & 0xff);
+    name.append((unicharacter & 0xff00) >> 8);
+  }
+  name.append(unicharacter & 0xff);
+  name.append((unicharacter & 0xff00) >> 8);
+  m_names[m_header.id] = VSDName(name, libvisio::VSD_TEXT_UTF16);
+}
+
 void libvisio::VSDParser::readTextField(WPXInputStream *input)
 {
   unsigned long initialPosition = input->tell();
@@ -1730,16 +1890,16 @@ void libvisio::VSDParser::readTextField(WPXInputStream *input)
   unsigned char tmpCode = readU8(input);
   if (tmpCode == 0xe8)
   {
-    int nameId = (int)readU32(input);
+    int nameId = readS32(input);
     input->seek(6, WPX_SEEK_CUR);
-    int formatStringId = (int)readU32(input);
+    int formatStringId = readS32(input);
     m_shape.m_fields.addTextField(m_header.id, m_header.level, nameId, formatStringId);
   }
   else
   {
     double numericValue = readDouble(input);
     input->seek(2, WPX_SEEK_CUR);
-    int formatStringId = (int)readU32(input);
+    int formatStringId = readS32(input);
 
     unsigned blockIdx = 0;
     unsigned length = 0;
@@ -1792,6 +1952,15 @@ void libvisio::VSDParser::readTextField(WPXInputStream *input)
   }
 }
 
+void libvisio::VSDParser::readMisc(WPXInputStream *input)
+{
+  unsigned char flags = readU8(input);
+  if (flags & 0x20)
+    m_shape.m_misc.m_hideText = true;
+  else
+    m_shape.m_misc.m_hideText = false;
+}
+
 libvisio::Colour libvisio::VSDParser::_colourFromIndex(unsigned idx)
 {
   if (idx < m_colours.size())
